==> zmailer-2.99.14-950422.tar.gz: zmailer-2.99.14/lib/nobody.c <==
/*
 *	Copyright 1988 by Rayan Zachariassen, all rights reserved.
 *	This will be free software, but only when it is finished.
 */

#include <stdio.h>
#include "hostenv.h"
#include "mailer.h"
#include <ctype.h>
#include <pwd.h>
#ifdef	__svr4__
#include <sys/param.h>
#endif

/*
 * Routine to initialize the 'nobody' variable to contain a nonprivileged uid.
 *
 * There are three choices:
 * 1. Whatever is specified by the NOBODY key in the /etc/zmailer.conf file
 * 2. The uid of the 'nobody' user if any
 * 3. -2 if that works
 * 4. MAX_SIGNED_SHORT otherwise
 */


char *nouser = "nobody";
#ifdef	UID_NOBODY
int nobody = UID_NOBODY;
#else
int nobody = -2;
#endif

static int nobodies[] = {
	0,
#ifdef	UID_NOBODY
	UID_NOBODY,
#else
	-2,
#endif
	29999,
	0
};

static int didnobody = 0;

int
getnobody()
{
	int i, factor = 1;
	struct passwd *pw;
	char *s;
	extern char *getzenv();

	if (didnobody)
		return nobody;

	if ((s = getzenv("NOBODY")) != NULL) {
		while (*s == '-')
			factor = -1, ++s;
		if (isdigit(*s) && atoi(s) != 0) {
			nobody = atoi(s) * factor;
			goto done;
		} else if (isalpha(*s) && (pw = getpwnam(s)) != NULL) {
			nobody = pw->pw_uid;
			goto done;
		}
	}

	if ((pw = getpwnam(nouser)) != NULL) {
		nobody = pw->pw_uid;
		goto done;
	}

	if (getuid() != 0)	/* it doesn't matter anyway */
		goto done;

	/* check that we can setuid(nobody) */
	i = -1;
	while (nobodies[++i] != 0) {
		if (
#ifdef	USE_SETREUID
		    setreuid(-1, nobodies[i])
#else	/* !USE_SETREUID */
#ifdef	USE_SETEUID
		    seteuid(nobodies[i])
#else	/* !USE_SETEUID */
		    setuid(nobodies[i])
#endif	/* USE_SETEUID */
#endif	/* USE_SETREUID */
					== 0)
			break;
	}

	if (nobodies[i] == 0) {
		perror("setuid");
		fprintf(stderr, "Cannot determine 'nobody' uid\n");
		exit(2);
	}
	if (
#ifdef	USE_SETREUID
	    setreuid(-1, 0)
#else	/* !USE_SETREUID */
#ifdef	USE_SETEUID
	    seteuid(0)
#else	/* !USE_SETEUID */
	    setuid(0)
#endif	/* !USE_SETEUID */
#endif	/* USE_SETREUID */
				    < 0 || getuid() != 0) {
		perror("setuid");
		fprintf(stderr, "Cannot reset root uid\n");
		exit(3);
	}

	nobody = nobodies[i];
done:
	didnobody = 1;
	return nobody;
	
}

==> zmailer-2.99.21-951110.tar.gz: zmailer-2.99.21/lib/nobody.c <==
/*
 *	Copyright 1988 by Rayan Zachariassen, all rights reserved.
 *	This will be free software, but only when it is finished.
 */

#include <stdio.h>
#include "hostenv.h"
#include "mailer.h"
#include <ctype.h>
#include <pwd.h>
#ifdef	__svr4__
#include <sys/param.h>
#endif

#ifdef	USE_SETUIDX	/* IBM AIXism */
# include <sys/id.h>
# ifdef  USE_SETREUID
#  define setreuid(x,y) setuidx(ID_REAL|ID_EFFECTIVE, y)
# else
#  define setuid(y) setuidx(ID_REAL|ID_EFFECTIVE, y)
# endif
#endif

/*
 * Routine to initialize the 'nobody' variable to contain a nonprivileged uid.
 *
 * There are three choices:
 * 1. Whatever is specified by the NOBODY key in the /etc/zmailer.conf file
 * 2. The uid of the 'nobody' user if any
 * 3. -2 if that works
 * 4. MAX_SIGNED_SHORT otherwise
 */


char *nouser = "nobody";
#ifdef	UID_NOBODY
int nobody = UID_NOBODY;
#else
int nobody = -2;
#endif

static int nobodies[] = {
	0,
#ifdef	UID_NOBODY
	UID_NOBODY,
#else
	-2,
#endif
	29999,
	0
};

static int didnobody = 0;

int
getnobody()
{
	int i, factor = 1;
	struct passwd *pw;
	char *s;
	extern char *getzenv();

	if (didnobody)
		return nobody;

	if ((s = getzenv("NOBODY")) != NULL) {
		while (*s == '-')
			factor = -1, ++s;
		if (isdigit(*s) && atoi(s) != 0) {
			nobody = atoi(s) * factor;
			goto done;
		} else if (isalpha(*s) && (pw = getpwnam(s)) != NULL) {
			nobody = pw->pw_uid;
			goto done;
		}
	}

	if ((pw = getpwnam(nouser)) != NULL) {
		nobody = pw->pw_uid;
		goto done;
	}

	if (getuid() != 0)	/* it doesn't matter anyway */
		goto done;

	/* check that we can setuid(nobody) */
	i = -1;
	while (nobodies[++i] != 0) {
		if (
#ifdef	USE_SETREUID
		    setreuid(-1, nobodies[i])
#else	/* !USE_SETREUID */
#ifdef	USE_SETEUID
		    seteuid(nobodies[i])
#else	/* !USE_SETEUID */
		    setuid(nobodies[i])
#endif	/* USE_SETEUID */
#endif	/* USE_SETREUID */
					== 0)
			break;
	}

	if (nobodies[i] == 0) {
		perror("setuid");
		fprintf(stderr, "Cannot determine 'nobody' uid\n");
		exit(2);
	}
	if (
#ifdef	USE_SETREUID
	    setreuid(-1, 0)
#else	/* !USE_SETREUID */
#ifdef	USE_SETEUID
	    seteuid(0)
#else	/* !USE_SETEUID */
	    setuid(0)
#endif	/* !USE_SETEUID */
#endif	/* USE_SETREUID */
				    < 0 || getuid() != 0) {
		perror("setuid");
		fprintf(stderr, "Cannot reset root uid\n");
		exit(3);
	}

	nobody = nobodies[i];
done:
	didnobody = 1;
	return nobody;
	
}
